/* your copyright here */

/* Enable debugging output */
#define LIBND_DEBUG 1

#include <libnd_template.h>


/* This is a basic libetdude protocol skeleton, tweak as you see fit!
 * Looking at the code of more complex protocol plugins, like those
 * for IP or TCP (in the libnetdude source tree) is highly recommended.
 *
 * At the shell prompt, you can always find out where protocols are
 * installed by typing "libnetdude-config --proto-dir".
 */

static LND_Protocol *template;

/* Plugin hook implementations: ---------------------------------------- */

const char *
name(void)
{
  return "Template Protocol Plugin";
}


const char *
description(void)
{
  return "A plugin providing Template support.";
}


const char *
author(void)
{
  return "You <your@email.foo>";
}


const char *
version(void)
{
  return "0.1";
}


/* This is the important part -- you need to create a new protocol structure
 * and return it. The library takes care of registering it properly.
 */
LND_Protocol *
init(void)
{
  template = libnd_proto_new("Template",
			     LND_PROTO_LAYER_LINK, /* Choose a layer here */
			     0xf000);              /* Choose a constant here */
			     		
  /* Now hook the functions into the protocol that your protocol
   * needs. Have a look at libnd_protocol.h and the manual for
   * your options. You do not need to supply functions for all
   * the methods, just those you require.
   */

  /* You will always need an initialization method! */
  template->init_packet     = libnd_template_init_packet;
  template->header_complete = libnd_template_header_complete;

  /* Depending on your protocol's needs, there will be more here. */
	     
  return template;
}


/* Protocol method implementations: ------------------------------------ */

/* A function to calculate whether a packet contains the full
 * protocol data. If not, it's good to not use your plugin but
 * instead use the raw packet data plugin, although you don't
 * have to.
 */
static gboolean
template_header_complete(const LND_Packet *packet, guchar *data)
{
  /* Input validation */
  if (!data)
    return FALSE;

  /* It depends on your protocol how you can figure out if you
   * have the entire protocol header present in the packet.
   * Here we assume that this is constant and TEMPLATE_HEADER_LENGTH
   * defines that size. This could be more complex if your header
   * has options etc.
   */
  return (data + TEMPLATE_HEADER_LENGTH <= libnd_packet_get_end(packet));
}

void       
libnd_template_init_packet(LND_Packet *packet, guchar *data, guchar *data_end)
{
  /* A protocol for the protocol nested inside yours, if any */
  LND_Protocol *payload_proto;

  /* Let's assume that struct template_header defines what your
   * protocol header looks like
   */  
  struct template_header *th;

  /* Using the enter/exit debugging macros is always a good idea at first */
  D_ENTER;

  /* If we don't have the full protocol data, let's fall back
   * on the raw protocol data. Much easier.
   */
  if (!template_header_complete(packet, data))
    {
      libnd_raw_proto_get()->init_packet(packet, data, data_end);

      /* If you use the enter/exit macros, always make sure you
       * balance them correctly!
       */
      D_RETURN;
    }

  /* Add a new protocol data segment to the packet, saying that this
   * segment is occupied by your protocol. Here we assume that the
   * data extend to the end given to us, this may not always be the
   * case.
   */
  libnd_packet_add_proto_data(packet, template, data, data_end);  

  /* The rest of this code assumes that your protocol can carry other
   * protocol data. If this is not the case, you do not have to figure
   * out the protocol to continue packet initialization with.
   */
  th = (struct template_header *) data;

  /* Investigate header fields in th to figure out what is the
   * next protocol. Let's assume there's a field proto_type in
   * th that defines this.
   */
  payload_proto =
    libnd_proto_registry_find(LND_PROTO_LAYER_NET, /* Choose one level up from your protocol */
			      th->proto_type);     /* May need byte order fixing if > 1 byte */

  /* This lookup can fail if no plugin supports the required
   * protocol -- in this case NULL is returned. You MUST check
   * for this case! 
   */
  if (! payload_proto)
    payload_proto = libnd_raw_proto_get();

  /* Pass initialization on to the next protocol. We need to adjust
   * the data pointers so that the next protocol "sees" only the
   * chunk that is relevant to it. Here we assume that our protocol
   * has no trailer. If this is not the case, we would have to
   * subtract the size occupied by that trailer from data_end.
   */
  payload_proto->init_packet(packet, data + TEMPLATE_HDR_LEN, data_end);

  /* And remember -- keep 'em in sync :) */
  D_RETURN;
}


/* This one should also be implemented. It tells users whether your
 * protocol's header is completely present in a packet, at a given
 * nesting level (think nested IP for example -- the first, outermost
 * occurrence is nesting level 0, the next (tunneled) one is nesting
 * level 1 etc).
 */
gboolean
libnd_template_header_complete(const LND_Packet *packet, guint nesting)
{
  guchar *data;

  if (!packet)
    return FALSE;

  /* This is actually really easy -- first try to obtain a pointer
   * to the spot in the packet data where your protocol starts at
   * the given nesting level. The result may be NULL here.
   */
  if (! (data = libnd_packet_get_data(packet, template, nesting)))
    return FALSE;

  return template_header_complete(packet, data);
  TOUCH(nesting);
}


/* Just a simple accessor function for your protocol */
LND_Protocol *
libnd_template_get(void)
{
  return template;
}

